/* StreamWriter.h */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef STREAM_WRITER_H
#define STREAM_WRITER_H

#include "StreamDataSender.h"
#include "StreamHttpParser.h"

#include "Helper/MetaData/MetaData.h"
#include "Helper/SayonaraClass.h"

#include <QByteArray>
#include <QHostAddress>
#include <memory>

class StreamWriter : public QObject, protected SayonaraClass {



	Q_OBJECT

	signals:
		void sig_new_connection(const QString& ip);
		void sig_disconnected(StreamWriter* sw);

	public:

		enum class Type : quint8 {
			Undefined,
			Standard,
			Invalid,
			Streaming
		};

		StreamWriter(QTcpSocket* socket, const QString& ip, const MetaData& md);
		virtual ~StreamWriter();


	private:
		StreamHttpParser*	_parser;
		StreamDataSender*	_sender;
		QTcpSocket*			_socket;

		bool				_dismissed; // after that, only trash will be sent
		bool				_send_data; // after that, no data at all will be sent

		QString				_stream_title;
		QString				_ip;

		StreamWriter::Type	_type;

		void reset();

	public:
		QString get_ip();
		int get_sd();

		void change_track(const MetaData& md);

		bool send_data(const uchar*, quint64 size);
		bool send_playlist();
		bool send_favicon();
		bool send_metadata();
		bool send_bg();
		bool send_html5();
		bool send_header(bool reject);

		StreamHttpParser::HttpAnswer parse_message();

		void disconnect();
		void dismiss();

		Type get_type();

	private slots:

		void socket_disconnected();
		void data_available();
};

typedef std::shared_ptr<StreamWriter> StreamWriterPtr;
#endif
