
/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "ElementOperator.h"
#include "ModifierCallbacks.h"
#include "PipelineModifier.h"

void finished(gpointer data);

ElementOperator::ElementOperator(PipelineModifier* modifier, GstElement* pipeline, ElementOperator::Mode mode){
	_pipeline = pipeline;
	_has_probe = false;
	_mode = mode;
	_modifier = modifier;

	switch(mode){
		case Mode::Add:
			_probe_callback = ModifierCallbacks::add_predecessor_blocked;
			break;
		case Mode::Remove:
			_probe_callback = ModifierCallbacks::remove_predecessor_blocked;
			break;
		case Mode::Replace:
			_probe_callback = ModifierCallbacks::replace_predecessor_blocked;
			break;
		default:
			_probe_callback = nullptr;
	}
}

void ElementOperator::start()
{
	if( _elems_of_interest.isEmpty() ){
		set_finished();
		return;
	}

	GstPad* block_pad = gst_element_get_static_pad(_predecessor, "src");
	gst_pad_add_probe(
		block_pad,
		GST_PAD_PROBE_TYPE_BLOCK_DOWNSTREAM,
		_probe_callback,
		this, // userdata
		finished
	);
}


void finished(gpointer data){
	ElementOperator* op = static_cast<ElementOperator*>(data);
	op->set_finished();
}

void ElementOperator::set_finished(){
	_modifier->set_finished();
}


QString ElementOperator::get_status(){
	return _status;
}

void ElementOperator::set_status(const QString & status){
	_status = status;
}


GstElement* ElementOperator::predecessor() const
{
	return _predecessor;
}

void ElementOperator::set_predecessor(GstElement* element){
	_predecessor = element;
}


GstElement* ElementOperator::successor() const
{
	return _successor;
}

void ElementOperator::set_successor(GstElement* element){
	_successor = element;
}


ElementVector ElementOperator::elems_of_interest() const
{
	return _elems_of_interest;
}

void ElementOperator::set_elems_of_interest(const ElementVector& elems_of_interest){
	_elems_of_interest = elems_of_interest;
}


GstElement* ElementOperator::elem_of_interest(int idx) const
{
	if(idx < 0 || idx >= _elems_of_interest.size()){
		return nullptr;
	}

	return _elems_of_interest[idx];
}

GstElement* ElementOperator::pipeline() const
{
	return _pipeline;
}


bool ElementOperator::has_probe() const
{
	return _has_probe;
}

void ElementOperator::set_probe(bool b){
	_has_probe = b;
}
