
/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef ELEMENTOPERATOR_H
#define ELEMENTOPERATOR_H

#include "PipelineHelper.h"


class PipelineModifier;
class ElementOperator
{

	public:
		enum class Mode : quint8
		{
			Remove=0,
			Add,
			Replace
		};


	private:

		Mode					_mode;
		GstElement*				_pipeline;				// the pipeline we are talking about

		GstElement*				_predecessor;			// the element before the action starts
		GstElement*				_successor;				// the element after the action starts
		ElementVector			_elems_of_interest;		// elements that have to be added/removed

		GstPadProbeCallback		_probe_callback;		// function where the party starts

		bool					_has_probe;				// indicates if a probe was applied to predecessor

		QString					_status;				// current status
		PipelineModifier*		_modifier;				// reference to modifier


	public:

		ElementOperator(PipelineModifier* modifier, GstElement* pipeline, ElementOperator::Mode mode);
		void start();

		GstElement* pipeline() const;

		void set_predecessor(GstElement* element);
		GstElement* predecessor() const ;

		void set_successor(GstElement* element);
		GstElement* successor() const;

		void set_elems_of_interest(const ElementVector& elems_of_interest);
		ElementVector elems_of_interest() const;

		GstElement* elem_of_interest(int idx) const;

		bool has_probe() const;
		void set_probe(bool b);

		void set_finished();
		QString get_status();
		void set_status(const QString&);

}; // class ELementOperator end;

#endif // ELEMENTOPERATOR_H
