/* PipelineHelper.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "PipelineHelper.h"

bool PipelineHelper::add_elements(GstElement* pipeline, const ElementVector& elements){


	bool success = true;
	for(GstElement* element : elements){
		bool tmp_success;
		gst_element_set_state(element, GST_STATE_PAUSED);
		tmp_success = gst_bin_add(GST_BIN(pipeline), element);
		success &= tmp_success;

		//sp_log(Log::Debug) << "Added " << GST_ELEMENT_NAME(element) << ": " << tmp_success;
	}

	return success;
}

bool PipelineHelper::add_element(GstElement *pipeline, GstElement *element){
	return add_elements(pipeline, ElementVector() << element);
}


bool PipelineHelper::remove_elements(GstElement* pipeline, const ElementVector& elements){

	bool success = true;
	for(GstElement* element : elements){
		gst_object_ref(element);
		gst_element_set_state(element, GST_STATE_NULL);
		success &= gst_bin_remove(GST_BIN(pipeline), element);

		//sp_log(Log::Debug) << "Removed " << GST_ELEMENT_NAME(element) << ": " << success;
	}

	return success;
}

bool PipelineHelper::remove_element(GstElement *pipeline, GstElement *element){
	return remove_elements(pipeline, ElementVector() << element);
}

bool PipelineHelper::link_elements(const ElementVector& elements){

	bool success = true;

	for(int i=elements.size() -1; i>=1; i--){
		bool tmp_success;

		if(elements[i] == nullptr){
			continue;
		}

		tmp_success = gst_element_link(elements[i-1], elements[i]);
		success &= tmp_success;
		//sp_log(Log::Debug) << "Linked " << GST_ELEMENT_NAME(elements[i-1]) << "->" << GST_ELEMENT_NAME(elements[i]) << ": " << tmp_success;
	}

	return success;
}


void PipelineHelper::apply_state(GstElement* pipeline, const ElementVector& elements){
	GstState target_state;
	gst_element_get_state(pipeline, &target_state, NULL, 0);

	for(GstElement* element : elements){
		//sp_log(Log::Debug) << "apply state " << gst_element_state_get_name(target_state) << " to " << GST_ELEMENT_NAME(element);
		gst_element_set_state(element, target_state);
	}
}

void PipelineHelper::apply_state(GstElement *pipeline, GstElement *element){
	apply_state(pipeline, ElementVector() << element);
}
