/* GSTPlaybackPipeline.h */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef GSTPLAYBACKPIPELINE_H_
#define GSTPLAYBACKPIPELINE_H_

#include "Components/Engine/AbstractPipeline.h"
#include "Components/Engine/Modifier/PipelineModifier.h"

#include <gst/app/gstappsink.h>


class GSTPlaybackPipeline : public GSTAbstractPipeline
{
	Q_OBJECT


public:
	GSTPlaybackPipeline(Engine* engine, QObject *parent=nullptr);
	virtual ~GSTPlaybackPipeline();

	virtual bool init(GstState state=GST_STATE_NULL);
	virtual bool set_uri(gchar* uri);

public slots:

	virtual void play() override;
	virtual void pause() override;
	virtual void stop() override;

	gint64 seek_rel(double percent, gint64 ref_ns);
	gint64 seek_abs(gint64 ns );

	void set_eq_band(QString band_name, double val);
	void unmute();

	void start_timer(qint64 ms);
	void set_speed(float f);

    void set_sr_path(QString session_path);


private:

	int					_vol;
	bool				_speed_active;
	float				_speed_val;
	QString				_sr_path;
	QTimer*				_timer;

	GstElement*			_audio_src;
	GstElement*			_audio_convert;
	GstElement*			_tee;

	GstElement*			_eq_queue;
	GstElement*			_equalizer;

	GstElement*			_volume;

	GstElement*			_audio_sink;

	GstElement*			_spectrum_queue;
	GstElement*			_spectrum;
	GstElement*			_spectrum_sink;

	GstElement*			_level_queue;
	GstElement*			_level;
	GstElement*			_level_sink;

	GstElement*			_lame_queue;
	GstElement*			_lame_converter;
	GstElement*			_lame_resampler;
	GstElement*			_lame;
	GstElement*			_lame_app_sink;
	GstElement*			_lame_fake_sink;

	GstElement*			_file_queue;
	GstElement*			_file_sink;
	GstElement*			_file_resampler;
	GstElement*			_file_fake_sink;
	GstElement*			_file_lame;
	GstElement*			_file_header;

	PipelineModifier* _pm_lame;
	PipelineModifier* _pm_level;
	PipelineModifier* _pm_spectrum;

    bool _seek(gint64 ns);
	bool tee_connect(GstPadTemplate* tee_src_pad_template,
					 GstElement* queue,
					 const QString& queue_name
	);

	virtual bool create_elements() override;
	virtual bool add_and_link_elements() override;
	virtual bool configure_elements() override;


protected slots:
	void _sl_broadcast_clients_changed();
	void _sl_vol_changed();
	void _sl_show_level_changed();
	void _sl_show_spectrum_changed();

	void lame_pipeline_modified(PipelineModifier::Status old_status);
};

#endif
