/* Playlist.h */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef PLAYLIST_H
#define PLAYLIST_H

#include "Helper/Helper.h"
#include "Helper/PlaylistMode.h"
#include "Helper/SayonaraClass.h"
#include "DatabaseAccess/CDatabaseConnector.h"

#include "PlaylistDBInterface.h"

#include <QString>
#include <QList>


class Playlist : public PlaylistDBInterface, protected SayonaraClass
{

	friend class PlaylistHandler;

public:

	enum class Type : quint8
	{
		Std=0,
		Stream
	};

protected:
	bool            _playlist_changed;
	int             _cur_play_idx;
	int				_playlist_idx;

	MetaDataList    _v_md;
	MetaDataList	_selected_tracks;
	PlaylistMode	_playlist_mode;
	PlaylistMode	_playlist_mode_backup;
	Type			_playlist_type;
	bool            _start_playing;


	Playlist(int idx, QString name="");
	virtual ~Playlist();

	virtual void play()=0;
	virtual void pause()=0;
	virtual void stop()=0;
	virtual void fwd()=0;
	virtual void bwd()=0;
    virtual void next()=0;
	virtual bool change_track(int idx)=0;

	virtual int create_playlist(const MetaDataList& v_md)=0;
    virtual void clear();

	virtual void metadata_changed(const MetaDataList& v_md_old, const MetaDataList& v_md_new)=0;

    virtual void move_track(const int, int tgt);
	virtual void move_tracks(const IdxList& lst, int tgt);

    virtual void delete_track(const int idx);
	virtual void delete_tracks(const IdxList& lst);

    virtual void insert_track(const MetaData& md, int tgt);
    virtual void insert_tracks(const MetaDataList& lst, int tgt);

    virtual void append_track(const MetaData& md);
    virtual void append_tracks(const MetaDataList& lst);

    virtual void replace_track(int idx, const MetaData& md);

	virtual void selection_changed(const IdxList&);


	virtual PlaylistMode playlist_mode_backup();
	virtual PlaylistMode playlist_mode_restore();

public:

	QStringList		toStringList() const;

	IdxList			find_tracks(int id) const;
	IdxList			find_tracks(const QString& filepath) const;

	bool			is_empty() const;


	Type			get_type() const;
	int				get_cur_track_idx() const;
	bool			get_cur_track(MetaData& md) const;
	int				get_idx() const;
	void			set_idx(int idx);	
	PlaylistMode	get_playlist_mode() const;
	void			set_playlist_mode(const PlaylistMode& mode);	
	MetaDataList	get_selected_tracks() const;

	virtual int				get_size() const override;
	virtual MetaDataList	get_playlist() const override;



};


#endif // PLAYLIST_H
