/* GUI_LastFM.cpp */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * GUI_LastFM.cpp
 *
 *  Created on: Apr 21, 2011
 *      Author: luke
 */

#include "GUI_LastFM.h"
#include "Helper/Helper.h"
#include "Components/StreamPlugins/LastFM/LastFM.h"


GUI_LastFM::GUI_LastFM(QWidget* parent) :
	SayonaraDialog(parent),
	Ui_GUI_LastFM_Dialog()
{

	setupUi(this);

	_lfm = LastFM::getInstance();
	logged_in(_lfm->is_logged_in());

	init();

	StringPair user_pw = _settings->get(Set::LFM_Login);
	tf_username->setText(user_pw.first);
	tf_password->setText(user_pw.second);

}


GUI_LastFM::GUI_LastFM(QWidget* parent, QString username, QString password) :
	GUI_LastFM(parent)
{
	tf_username->setText(username);
	tf_password->setText(password);
}

void GUI_LastFM::init(){

	bool enabled = _settings->get(Set::LFM_Active);

	cb_activate->setChecked(enabled);
	setLFMActive(enabled);

	connect(btn_ok, &QPushButton::clicked, this, &GUI_LastFM::btn_ok_clicked);
	connect(btn_login, &QPushButton::clicked, this, &GUI_LastFM::btn_login_clicked);
	connect(cb_activate, &QCheckBox::toggled, this, &GUI_LastFM::cb_activate_toggled);
	connect(_lfm, &LastFM::sig_last_fm_logged_in, this, &GUI_LastFM::logged_in);

	hide();
}


GUI_LastFM::~GUI_LastFM() {

}


void GUI_LastFM::language_changed() {

	retranslateUi(this);
}

void GUI_LastFM::btn_login_clicked(){

	if(tf_username->text().length() < 3) return;
	if(tf_password->text().length() < 3) return;

	StringPair user_pw;
	user_pw.first = tf_username->text();
	user_pw.second = tf_password->text();

	_settings->set(Set::LFM_Login, user_pw);

	_lfm->psl_login();
}

void GUI_LastFM::btn_ok_clicked() {

	StringPair user_pw;
	user_pw.first = tf_username->text();
	user_pw.second = tf_password->text();

	_settings->set(Set::LFM_Login, user_pw);

	if( tf_username->text().length() >= 3 &&
		tf_password->text().length() >= 3 )
	{
		_lfm->psl_login();

		_settings->set( Set::LFM_Active, cb_activate->isChecked() );
	}

	close();
}


void GUI_LastFM::show_win() {

	QString user, password;
	LastFM::get_login(user, password);

	if (user.size() > 0) {
		tf_username->setText(user);
		tf_password->setText(password);
	}

    else {
		tf_username->setPlaceholderText("Enter Usename");
		tf_password->setPlaceholderText("Enter Password");
    }

	show();
}


void GUI_LastFM::cb_correct_id3_toggled(bool checked) {

	_settings->set(Set::LFM_Corrections, checked);
}

void GUI_LastFM::setLFMActive(bool enabled) {
	tf_username->setEnabled(enabled);
	tf_password->setEnabled(enabled);
}

void GUI_LastFM::cb_activate_toggled(bool b) {
	setLFMActive(b);

}

void GUI_LastFM::logged_in(bool success){
	if(success){
		lab_status->setText(tr("Logged in"));
	}

	else{
		lab_status->setText(tr("Not logged in"));
	}

}
