/* LibraryView.h */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * MyListView.h
 *
 *  Created on: Jun 26, 2011
 *      Author: luke
 */

#ifndef MYTABLEVIEW_H_
#define MYTABLEVIEW_H_

#include "GUI/Library/MyColumnHeader.h"

#include "GUI/Library/model/LibraryItemModel.h"
#include "GUI/ContextMenu/ContextMenu.h"
#include "GUI/SearchableWidget/MiniSearcher.h"
#include "GUI/SearchableWidget/SearchableTableView.h"

#include "Helper/globals.h"
#include "Helper/Helper.h"
#include "Helper/CustomMimeData.h"

#include <QDropEvent>
#include <QMouseEvent>
#include <QStringList>
#include <QHeaderView>
#include <QAction>
#include <QIcon>
#include <QLineEdit>
#include <QScrollBar>
#include <QFont>
#include <QDrag>
#include <QApplication>

class LibraryView : public SearchableTableView{

	Q_OBJECT

signals:

	void sig_columns_changed(const BoolList&);

	void sig_middle_button_clicked(const QPoint&);
	void sig_all_selected();
	void sig_info_clicked();
	void sig_edit_clicked();
	void sig_delete_clicked();
	void sig_play_next_clicked();
	void sig_append_clicked();
	void sig_refresh_clicked();
	void sig_sortorder_changed(SortOrder);

	void sig_no_disc_menu();
	void sig_import_files(const QStringList&);
	void sig_double_clicked(const IdxList&);
	void sig_sel_changed(const IdxList&);


protected slots:
	virtual void rc_header_menu_changed(bool b=true);
	virtual void rc_menu_show(const QPoint&);
	virtual void sort_by_column(int);
	virtual void drag_deleted();

public:
	LibraryView(QWidget* parent=nullptr);
	virtual ~LibraryView();

	virtual void rc_header_menu_init(const BoolList& lst);
	virtual void set_table_headers(const QList<ColumnHeader>& headers, SortOrder sorting);

	virtual void save_selections();

	using QTableView::setModel;
	virtual void setModel(LibraryItemModel* model);

	// entries are specified in ContextMenu.h
	virtual void set_rc_menu(int entries);


protected:
	// Events implemented in LibraryViewEvents.cpp
	virtual void mousePressEvent(QMouseEvent* event);
	virtual void mouseReleaseEvent(QMouseEvent* event);
	virtual void mouseMoveEvent(QMouseEvent* event);
	virtual void mouseDoubleClickEvent(QMouseEvent *event);
	virtual void keyPressEvent(QKeyEvent* event);
	virtual void dropEvent(QDropEvent* event);
	virtual void dragEnterEvent(QDragEnterEvent *event);
	virtual void dragMoveEvent(QDragMoveEvent *event);
	virtual void resizeEvent(QResizeEvent *event);

	virtual void selectionChanged ( const QItemSelection & selected, const QItemSelection & deselected );
	virtual void rc_menu_init();

	virtual void do_drag();

	virtual void set_col_sizes();


protected:

	QDrag*						_drag;
	QPoint						_drag_pos;

	QMenu*						_rc_header_menu;
	QList<ColumnHeader>			_table_headers;
	QList<QAction*>				_header_rc_actions;

	ContextMenu*				_rc_menu;

	SortOrder					_sort_order;

	LibraryItemModel*			_model;

	bool						_cur_filling;

	virtual int get_min_selected();


public:
	template < typename T, template <typename Elem> class Cont >
	void fill(const Cont<T>& input_data){

		IdxList idx_list_int;
		int size;

		_cur_filling = true;

		size = input_data.size();
		_model->remove_all_and_insert(size);

		for(int row=0; row < size; row++) {
			QModelIndex idx;
			QVariant var_data;

			if(_model->is_selected(input_data[row].id)){
				idx_list_int << row;
			}

			idx = _model->index(row, 1);
			var_data = T::toVariant( input_data[row] );
			_model->setData(idx, var_data, Qt::EditRole );
		}

		_model->clear_selections();
		select_rows(idx_list_int, 0, _model->columnCount() - 1);

		_cur_filling = false;
	}
};

#endif /* MYLISTVIEW_H_ */
