/* GUI_Notifications.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "GUI_Notifications.h"

GUI_Notifications::GUI_Notifications(QWidget *parent) :
	SayonaraDialog(parent),
	Ui_GUI_Notification()
{
	setupUi(this);

	_notification_handler = NotificationHandler::getInstance();

	int timeout = _settings->get(Set::Notification_Timeout);
	int active = _settings->get(Set::Notification_Show);

	sb_timeout->setValue(timeout);
	cb_activate->setChecked(active);

	connect(btn_ok, &QPushButton::clicked, this, &GUI_Notifications::ok_clicked);
	connect(_notification_handler, &NotificationHandler::sig_notifications_changed,
			this, &GUI_Notifications::notifications_changed);
}

GUI_Notifications::~GUI_Notifications() {
}

void GUI_Notifications::language_changed(){
	retranslateUi(this);
}


void GUI_Notifications::notifications_changed(){
	QVector<AbstractNotificator*> notifications = _notification_handler->get_notificators();

	combo_notification->clear();
	for(const AbstractNotificator* notification : notifications){
		combo_notification->addItem(notification->get_name());
	}

	combo_notification->setCurrentIndex(_notification_handler->get_cur_idx());
}

void GUI_Notifications::ok_clicked() {

	bool active =       cb_activate->isChecked();
	int timeout =       sb_timeout->value();
	QString cur_text =  combo_notification->currentText();
    
	_settings->set(Set::Notification_Name, cur_text);
	_settings->set(Set::Notification_Timeout, timeout);
	_settings->set(Set::Notification_Show, active);

	_notification_handler->notificator_changed(cur_text);


    close();
}
