/* GUI_PlayerEvents.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "GUI_Player.h"


void GUI_Player::moveEvent(QMoveEvent *e) {

	QMainWindow::moveEvent(e);

	QPoint p= this->pos();
	_settings->set(Set::Player_Pos, p);
}


void GUI_Player::resizeEvent(QResizeEvent* e) {

	QMainWindow::resizeEvent(e);

	if(ui_playlist){
		ui_playlist->resize(playlist_widget->size());
	}

	if(library_widget->isVisible()) {

		if( ui_libpath && ui_libpath->isVisible() ){
			ui_libpath->resize(library_widget->size());
		}

		else{
			_cur_library->resize(library_widget->size());
		}
	}

	QSize sz = plugin_widget->size();

	bool is_maximized = _settings->get(Set::Player_Maximized);
	bool is_fullscreen = _settings->get(Set::Player_Fullscreen);

	if(is_maximized) {
		_settings->set(Set::Player_Fullscreen, false);
	}

	set_album_label();
	lab_artist->setText(Helper::elide_text(_md.artist, lab_artist, 1));
	lab_title->setText(Helper::elide_text(_md.title, lab_title, 2));

	if(_pph){
		_pph->resize(sz);
	}

	if( !is_maximized &&
		!this->isMaximized() &&
		!is_fullscreen &&
		!this->isFullScreen())
	{
		_settings->set(Set::Player_Size, this->size());
	}

	update();
}


void GUI_Player::keyPressEvent(QKeyEvent* e) {

	e->accept();

	switch (e->key()) {

		case (Qt::Key_F9):

		if(!isMaximized())
			this->showMaximized();
		else this->showNormal();
			break;

		case (Qt::Key_F10):
			action_Dark->setChecked(!action_Dark->isChecked());
			break;

		case (Qt::Key_F11):
			show_fullscreen_toggled(!this->isFullScreen());
			break;

		default:
			break;
	}
}

void GUI_Player::showEvent(QShowEvent *e){

	Q_UNUSED(e);

	bool maximized = _settings->get(Set::Player_Maximized);

	if(maximized){
		showMaximized();
	}

	else{
		showNormal();
	}

	_cur_library->resize(library_widget->size());
	ui_playlist->resize(playlist_widget->size());
}



void GUI_Player::closeEvent(QCloseEvent* e) {

	bool min_to_tray = _settings->get(Set::Player_Min2Tray);

	if(min_to_tray && !this->isHidden()) {
		e->ignore();
		this->hide();
	}


	_settings->set(Set::Player_Maximized, this->isMaximized());
	_settings->set(Set::Player_Fullscreen, this->isFullScreen());
	_settings->set(Set::Player_Pos, this->pos());

}
